"""
这里定义一些百度翻译返回的异常类型
API 参考：https://fanyi-api.baidu.com/doc/21
"""


class BaiduBaseException(Exception):
    """基类，代码 52000 是成功代码"""

    def __init__(self, code: str = "52000", msg: str = f"发生未知错误..."):
        self.code = code
        self.msg = msg

    def __str__(self):
        return f"错误代码：{self.code}\t错误信息：{self.msg}"


class BaiduTimeoutException(BaiduBaseException):
    """请求超时"""

    def __init__(self, code: str = "52001", msg: str = "请求超时，请重试 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduInternalError(BaiduBaseException):
    """系统错误"""

    def __init__(self, code: str = "52002", msg: str = "系统错误，请重试 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduUnauthorizeException(BaiduBaseException):
    """未授权用户"""

    def __init__(self, code: str = "52003", msg: str = "请检查 appid 是否正确或者服务是否开通 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduMissingArgException(BaiduBaseException):
    """缺少必填参数"""

    def __init__(self, code: str = "54000", msg: str = "请检查是否少传参数 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduSignError(BaiduBaseException):
    """签名错误"""

    def __init__(self, code: str = "54001", msg: str = "请检查您的签名生成方法 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduFrequencyLimitException(BaiduBaseException):
    """访问频率受限"""

    def __init__(self, code: str = "54003", msg: str = "请降低您的调用频率，或进行身份认证后切换为高级版/尊享版 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduNoBalanceException(BaiduBaseException):
    """账户余额不足"""

    def __init__(self, code: str = "54004",
                 msg: str = "请前往 https://api.fanyi.baidu.com/api/trans/product/desktop 为账户充值 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduLongQueryException(BaiduBaseException):
    """长 query 请求频繁"""

    def __init__(self, code: str = "54005", msg: str = "请降低长query的发送频率，3s后再试 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduIllegalClientIPException(BaiduBaseException):
    """客户端IP非法"""

    def __init__(self, code: str = "58000",
                 msg: str = "检查个人资料里填写的IP地址是否正确，可前往 https://api.fanyi.baidu.com/access/0/3 修改 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduNotSupportLanguageException(BaiduBaseException):
    """译文语言方向不支持"""

    def __init__(self, code: str = "58001", msg: str = "检查译文语言是否在语言列表里 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduServiceClosedException(BaiduBaseException):
    """服务当前已关闭"""

    def __init__(self, code: str = "58002", msg: str = "请前往 https://api.fanyi.baidu.com/choose 开启服务 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


class BaiduServiceUnavailableException(BaiduBaseException):
    """认证未通过或未生效"""

    def __init__(self, code: str = "90107", msg: str = "请前往 https://api.fanyi.baidu.com/myIdentify 开启服务 ..."):
        super().__init__(code, msg)
        self.code = code
        self.msg = msg


EXCEPTIONS = {
    "52000": BaiduBaseException,
    "52001": BaiduTimeoutException,
    "52002": BaiduInternalError,
    "52003": BaiduUnauthorizeException,
    "54000": BaiduMissingArgException,
    "54001": BaiduSignError,
    "54003": BaiduFrequencyLimitException,
    "54004": BaiduNoBalanceException,
    "54005": BaiduLongQueryException,
    "58000": BaiduIllegalClientIPException,
    "58001": BaiduNotSupportLanguageException,
    "58002": BaiduServiceClosedException,
    "90107": BaiduServiceUnavailableException,
}

if __name__ == "__main__":
    from numpy.random import choice

    code = choice(list(EXCEPTIONS.keys()))
    print(code)
    raise EXCEPTIONS[code]



from .config import Config
from .data_source import translate_msg, LANGUAGES
from typing import Tuple, Any

from nonebot import on_regex
from nonebot.params import RegexGroup

translate = on_regex(r"^(.*)?翻(.*?)\s(.*)?$", priority=5, block=True)


@translate.handle()
async def _(reg_group: Tuple[Any, ...] = RegexGroup()):
    _query = reg_group[-1].strip()  # 翻译内容
    _from, _to = reg_group[0], reg_group[1]
    if _from and _to:
        _from_to = [_from, _to]
    else:
        await translate.finish(f"指令打错啦！请输入“x翻x 内容”\n其中x可以为: {', '.join(LANGUAGES.keys())}", at_sender=True)

    if len(_query) > 2000:
        await translate.finish("翻译过长！请不要超过2000字", at_sender=True)
    await translate.send(await translate_msg(_from_to, _query), at_sender=True)



import setuptools


with open("README.md", "r", encoding='utf-8') as f:
    long_description = f.read()


setuptools.setup(
    name="nonebot_plugin_baidutranslate",
    version="0.1.0",
    author="Number_Sir",
    author_email="Number_Sir@126.com",
    keywords=["pip", "nonebot2", "nonebot", "nonebot_plugin"],
    description="""基于 OneBot 适配器的 NoneBot2 百度翻译插件, 可以调用百度翻译 API 实现常用语种之间的互译""",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/NumberSir/nonebot_plugin_baidutranslate",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    platforms="any",
    install_requires=['nonebot-adapter-onebot>=2.0.0-beta.1', 'nonebot2>=2.0.0-beta.1', 'httpx>=0.22.0'],
    python_requires=">=3.7"
)


import hashlib

import httpx
from nonebot import get_driver

from .config import Config
from .utils import EXCEPTIONS, LANGUAGES
from nonebot import logger

baidu_config = Config.parse_obj(get_driver().config.dict())


async def translate_msg(_from_to, _query):
    lang = parse_language(_from_to)
    if lang is None:
        return f"指令打错啦！请输入“x翻x 内容”\n其中x可以为: {', '.join(LANGUAGES.keys())}"

    [_from, _to] = lang
    _appid = baidu_config.appid
    _salt = baidu_config.salt
    _key = baidu_config.key
    _sign = f"{_appid}{_query}{_salt}{_key}"
    _sign = hashlib.md5(bytes(_sign, 'utf-8')).hexdigest()

    async with httpx.AsyncClient() as client:
        success = False
        for times in range(5):
            try:
                url = "https://fanyi-api.baidu.com/api/trans/vip/translate"
                params = {
                    "q": _query,
                    "from": _from,
                    "to": _to,
                    "appid": _appid,
                    "salt": _salt,
                    "sign": _sign,
                }

                json_data = await client.post(url, params=params)
            except Exception as e:
                logger.warning(f"第{times + 1}次连接失败... {type(e)}: {e}")
                continue
            else:
                success = True
                break
    if not success:
        return f"QAQ，连接失败了...请重试..."

    json_data = json_data.json()
    logger.debug(f"结果: {json_data}")
    if "error_code" not in json_data.keys():
        _result = json_data['trans_result'][0]
        return f"\n原文：{_query}\n译文：{_result['dst']}"

    if json_data['error_code'] != "52000":
        raise EXCEPTIONS[json_data['error_code']]


def parse_language(_from_to):
    if len(_from_to) != 2:
        return None
    _rst = ["", ""]
    for idx, val in enumerate(_from_to):
        try:
            _rst[idx] = LANGUAGES[val]
        except KeyError:
            continue
    return _rst
